package packed

import (
	"archive/zip"
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"image"
	"image/jpeg"
	"image/png"
	"io/ioutil"
	"os"
	"path"
	"strings"
	"time"

	"ppt_server/app/model"
	"ppt_server/library"

	"github.com/gogf/gf/frame/g"
	"github.com/xxjwxc/gowp/workpool"
)

// 解析URL返回基本信息
type ObtainFile struct {
	r          *model.FileUploadRequest    //客户端请求参数
	bucket     *ossSdk                     // oss上传bucket
	ok         chan *model.SuccessResponse // 上传传课件状态
	err        chan error                  // 上传课件失败Error
	fileSize   int64                       // 课件大小
	pageCount  int                         // 课件图片数量
	parserFile *file                       // 课件对象
}

const (
	ParserFileLink     = "http://doc.offcncloud.com/"
	OssIntranetLink    = "xiaoyu-live.oss-cn-beijing-internal.aliyuncs.com"
	UploadFileLinkHost = "desktop.offcncloud.com"
)

func ParserURL(r *model.FileUploadRequest, errChan chan error,
	ok chan *model.SuccessResponse) (*ObtainFile, error) {

	f, err := NewFile(r.Url, r.Name)
	if err != nil {
		g.Log().Async().Error(err)
		return nil, err
	}

	bucket, err := NewOss(g.Cfg().GetString("oss.bucket"))
	if err != nil {
		g.Log().Async().Error(err)
		return nil, err
	}
	file := &ObtainFile{
		bucket:     bucket,
		ok:         ok,
		err:        errChan,
		parserFile: f,
		r:          r,
	}

	return file, nil
}

//从web365获取上传文件信息
func (f *ObtainFile) fileInfo() error {
	fileBytes, err := library.Get(f.parserFile.fileInfoURL)
	if err != nil {
		return err
	}

	var fileInfo model.UploadFileInfo
	if err := json.Unmarshal(fileBytes, &fileInfo); err != nil {
		return err
	}

	if fileInfo.FileSize != 0 {
		f.fileSize = fileInfo.FileSize
	}

	switch {
	case fileInfo.SlideCount != 0:
		f.pageCount = fileInfo.SlideCount
	case fileInfo.PageCount != 0:
		f.pageCount = fileInfo.PageCount
	case fileInfo.SheetCount != 0:
		f.pageCount = fileInfo.SheetCount
	}

	return nil
}

func (f *ObtainFile) job(file *zip.File) ([]interface{}, error) {
	fd, err := file.Open()
	if err != nil {
		return nil, err
	}

	defer fd.Close()

	fileBytes := []byte{}
	fileBytes, _ = ioutil.ReadAll(fd)
	// 获取图片信息reader
	imageReader := ioutil.NopCloser(bytes.NewBuffer(fileBytes))
	// 上传图片reader
	fileReader := ioutil.NopCloser(bytes.NewBuffer(fileBytes))

	var img image.Image
	fileSuffix := path.Ext(file.Name) //获取文件后缀
	switch fileSuffix {
	case ".png":
		img, _ = png.Decode(imageReader)
	case ".jpeg":
		img, _ = jpeg.Decode(imageReader)
	default:
		return nil, errors.New("image ext is not found")
	}
	if img == nil {
		return nil, errors.New("Image invalid")
	}

	var filenameOnly string
	filenameOnly = strings.TrimRight(file.Name, fileSuffix)
	filenameOnly = strings.Replace(filenameOnly, "p", "", -1)
	fileName := fmt.Sprintf("%05s", filenameOnly) + fileSuffix

	info := img.Bounds()
	var imageInfo []interface{}
	imageInfo = append(imageInfo, fileName)
	imageInfo = append(imageInfo, info.Max.X)
	imageInfo = append(imageInfo, info.Max.Y)

	g.Log().Async().Infof("Upload picture file: %s Success", fileName)

	return imageInfo, f.bucket.Upload(f.parserFile.uploadPath+"images/"+fileName, fileReader)
}

func (f *ObtainFile) uploadPng(zipFileAddress string) ([][]interface{}, error) {
	reader, err := zip.OpenReader(zipFileAddress)
	if err != nil {
		return nil, err
	}
	defer reader.Close()
	pool := workpool.New(100)
	var imageInfos [][]interface{}
	for _, file := range reader.File {
		pool.DoWait(func() error {
			var err error
			var imageInfo []interface{}
			for i := 0; i < 3; i++ {
				if imageInfo, err = f.job(file); err == nil {
					imageInfos = append(imageInfos, imageInfo)
					break
				}
			}
			return err
		})
	}
	if err = pool.Wait(); err != nil {
		return nil, err
	}

	return imageInfos, nil
}

// 上传packed.json

func (f *ObtainFile) uploadPacked(imagesInfos [][]interface{}) error {
	packed := model.Packed{
		FileName:      f.parserFile.name,
		Hash:          f.parserFile.contentHash,
		OssImagesPath: f.parserFile.uploadPath + "images/",
		ImageInfos:    imagesInfos,
	}

	for i := 1; i <= f.pageCount; i++ {
		packed.ImagesName = append(packed.ImagesName, fmt.Sprintf("%05d", i)+".png")
	}
	data, err := json.Marshal(packed)
	if err != nil {
		return err
	}
	return f.bucket.Upload(f.parserFile.uploadPath+"packed.json", bytes.NewReader(data))
}

// 进行上传操作
func (f *ObtainFile) Worker(err error) {

	if err != nil {
		return
	}
	// 解析课件获取课件信息
	if err = f.fileInfo(); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s file info complete", f.parserFile.name)

	downloadZipLink := fmt.Sprintf("%s?info=1&words=%v&ssl=1&furl=%s/%s",
		ParserFileLink,
		f.pageCount,
		g.Cfg().GetString("oss.url"),
		f.parserFile.dataPath)

	// 下载ZIP包

	if err = library.Download(downloadZipLink, f.parserFile.tempZipPath); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s Zip Donwload Complete", f.parserFile.name)

	// 解析ZIP包并上传图片到OSS
	imagesInfos, err := f.uploadPng(f.parserFile.tempZipPath)
	if err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}
	g.Log().Async().Infof("file: %s Upload picture Complete!", f.parserFile.name)
	// 上传packed.json到oss
	if err := f.uploadPacked(imagesInfos); err != nil {
		g.Log().Async().Error(err)
		f.err <- err
		return
	}

	g.Log().Async().Infof("filename == %s upload success!", f.parserFile.name)
	// 清理zip包
	_ = os.Remove(f.parserFile.tempZipPath)

	res := &model.SuccessResponse{
		Name:        f.parserFile.name,
		NameHash:    f.parserFile.nameHash,
		ContentHash: f.parserFile.contentHash,
		DataPath:    f.parserFile.dataPath,
		DocType:     f.parserFile.docType,
		MineType:    f.parserFile.mineType,
		FileSize:    f.fileSize,
		PageCount:   f.pageCount,
		Link:        strings.Replace(f.parserFile.url, OssIntranetLink, UploadFileLinkHost, -1),
		Private:     "1",
		Time:        time.Now().Format("2006-01-02 15:04:05"),
		Nickanem:    f.r.Nickanem,
		Uuid:        f.r.Uuid,
	}
	switch f.r.Type {
	case 1:
		res.IsMaterial = 1
	case 2:
		res.IsCourseware = 1
	case 3:
		res.IsTitlebook = 1
	}
	f.ok <- res
}
